/***************************************************************************
 * menu_data.h  -  header for the corresponding cpp file
 *
 * Copyright (C) 2004 - 2008 Florian Richter
 ***************************************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MENU_DATA_H__
#define __MENU_DATA_H__

#include "../core/globals.h"
#include "../gui/menu.h"
#include "../gui/hud.h"

/* *** *** *** *** *** *** *** cMenu_Base *** *** *** *** *** *** *** *** *** *** */

class cMenu_Base
{
public:
	cMenu_Base( void );
	virtual ~cMenu_Base( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );
	void Draw_End( void );

	// Exit
	virtual void Exit( void );

	// gui layout filename
	string layout_file;
	// CEGUI window
	Window *guiwindow;

	// if button/key action
	bool action;

	// menu position
	float menu_posy;
	// default text color
	Color text_color;
	// value text color
	Color text_color_value;

	// current menu sprites
	typedef vector<cHudSprite *> HudSpriteList;
	HudSpriteList drawlist;
};

/* *** *** *** *** *** *** *** cMenu_Main *** *** *** *** *** *** *** *** *** *** */

class cMenu_Main : public cMenu_Base
{
public:
	cMenu_Main( void );
	virtual ~cMenu_Main( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Exit
	virtual void Exit( void );
};

/* *** *** *** *** *** *** *** cMenu_Start *** *** *** *** *** *** *** *** *** *** */

class cMenu_Start : public cMenu_Base
{
public:
	cMenu_Start( void );
	virtual ~cMenu_Start( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Exit
	virtual void Exit( void );

	// Get all levels from the given directory
	void Get_Levels( string dir, colour color );

	/* Load the Selected Listbox item
	 * and exit if successful
	*/
	void Load_Selected( void );

	/* Load the Selected World
	 * and exit if successful
	*/
	void Load_World( string level_name );
	/* Load the Selected Level
	 * and exit if successful
	*/
	bool Load_Level( string name );

	// CEGUI events
	bool Window_Keydown( const EventArgs &event ); // key down

	bool World_Select( const EventArgs &event ); // world selected
	bool World_Select_final_list( const EventArgs &event ); // world selected for entering

	bool Level_Select( const EventArgs &event ); // level selected
	bool Level_Select_Final_List( const EventArgs &event ); // level selected for entering

	bool Button_Level_New_Clicked( const EventArgs &event ); // level new button
	bool Button_Level_Edit_Clicked( const EventArgs &event ); // level edit button
	bool Button_Level_Delete_Clicked( const EventArgs &event ); // level delete button
	bool Button_Enter_Clicked( const EventArgs &event ); // enter button
	bool Button_Back_Clicked( const EventArgs &event ); // back button
};

/* *** *** *** *** *** *** *** cMenu_Options *** *** *** *** *** *** *** *** *** *** */

class cMenu_Options : public cMenu_Base
{
public:
	cMenu_Options( void );
	virtual ~cMenu_Options( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Exit
	virtual void Exit( void );
};

/* *** *** *** *** *** *** *** cMenu_Options_Game *** *** *** *** *** *** *** *** *** *** */

class cMenu_Options_Game : public cMenu_Options
{
public:
	cMenu_Options_Game( void );
	virtual ~cMenu_Options_Game( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// CEGUI events
	bool Always_Run_Select( const EventArgs &event ); // always run option selected
	bool Camera_Hor_Select( const EventArgs &event ); // Camera Horizontal Value changed
	bool Camera_Ver_Select( const EventArgs &event ); // Camera Vertical Value changed
};

/* *** *** *** *** *** *** *** cMenu_Options_Video *** *** *** *** *** *** *** *** *** *** */

class cMenu_Options_Video : public cMenu_Options
{
public:
	cMenu_Options_Video( void );
	virtual ~cMenu_Options_Video( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// CEGUI events
	bool Res_Select( const EventArgs &event ); // resolution option selected
	bool Bpp_Select( const EventArgs &event ); // bpp option selected
	bool Fullscreen_Select( const EventArgs &event ); // fullscreen option selected
	bool Vsync_Select( const EventArgs &event ); // vsync option selected

	// Fullscreen Image
	cHudSprite *fullscreen;

	// video settings
	unsigned int vid_w;
	unsigned int vid_h;
	unsigned int vid_bpp;
	bool vid_fullscreen;
	bool vid_vsync;
};

/* *** *** *** *** *** *** *** cMenu_Options_Audio *** *** *** *** *** *** *** *** *** *** */

class cMenu_Options_Audio : public cMenu_Options
{
public:
	cMenu_Options_Audio( void );
	virtual ~cMenu_Options_Audio( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Audio Hz option selected event
	bool Audio_Hz_Select( const EventArgs &event );
	// music option selected event
	bool Music_Select( const EventArgs &event );
	// music volume changed event
	bool Music_Vol_Changed( const EventArgs &event );
	// sound option selected event
	bool Sound_Select( const EventArgs &event );
	// sound volume changed event
	bool Sound_Vol_Changed( const EventArgs &event );

	// Audio
	cHudSprite *music;
	cHudSprite *sounds;
};

/* *** *** *** *** *** *** *** cMenu_Options_Controls *** *** *** *** *** *** *** *** *** *** */

class cMenu_Options_Controls : public cMenu_Options
{
public:
	cMenu_Options_Controls( void );
	virtual ~cMenu_Options_Controls( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	/* Build the shortcut list
	 * joystick : if true build it for joystick
	*/
	void Build_Shortcut_List( bool joystick = 0 );
	/* Set the given Shortcut
	 * and exit if successful
	 * joystick : if true set it for joystick
	*/
	void Set_Shortcut( string shortcut_name, bool joystick = 0 );

	// Select given Joystick
	void Joy_Default( unsigned int index );
	// Disable Joystick
	void Joy_Disable( void );

	// keyboard listbox item double clicked event
	bool Keyboard_List_Double_Click( const EventArgs &event );
	// keyboard scroll speed value changed event
	bool Keyboard_Slider_Scroll_Speed_Changed( const EventArgs &event );
	// joystick name click event
	bool Joystick_Name_Click( const EventArgs &event );
	// joystick analog jump click event
	bool Joystick_Analog_Jump_Click( const EventArgs &event );
	// joystick name option selected event
	bool Joystick_Name_Select( const EventArgs &event );
	// joystick analog jump option selected event
	bool Joystick_Analog_Jump_Select( const EventArgs &event );
	// joystick axis horizontal changed event
	bool Joystick_Spinner_Axis_Hor_Changed( const EventArgs &event );
	// joystick axis vertical changed event
	bool Joystick_Spinner_Axis_Ver_Changed( const EventArgs &event );
	// joystick listbox item double clicked event
	bool Joystick_List_Double_Click( const EventArgs &event );


	// Gamepad Image
	cHudSprite *gamepad;
};

/* *** *** *** *** *** *** *** cMenu_Savegames *** *** *** *** *** *** *** *** *** *** */

class cMenu_Savegames : public cMenu_Base
{
public:
	cMenu_Savegames( bool ntype_save );
	virtual ~cMenu_Savegames( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Exit
	virtual void Exit( void );

	void Update_Load( void );
	void Update_Save( void );

	// Set Savegame Description
	string Set_Save_Description( unsigned int save_slot );
	// Update Savegame Descriptions
	void Update_Saved_Games_Text( void );

	// Savegame images
	HudSpriteList savegame_temp;

	// if save menu
	bool type_save;
};

/* *** *** *** *** *** *** *** cMenu_Credits *** *** *** *** *** *** *** *** *** *** */

class cMenu_Credits : public cMenu_Base
{
public:
	cMenu_Credits( void );
	virtual ~cMenu_Credits( void );

	virtual void Init( void );
	virtual void Init_GUI( void );
	virtual void Update( void );
	virtual void Draw( void );

	// Exit
	virtual void Exit( void );
};

#endif

